function [TelList, mirrorMatrix] = MakeMultiTelescope(N, q1, q2)


if ~exist('q1', 'var') 
    q1 = 1i*0.37;
end

if ~exist('q2', 'var') 
    q2 = 1i*pi*3.3e-3^2/1.064e-6;  % right for POP
end

if ~exist('N', 'var')
    N = 2;  
end

xMin = 0.1;
xMax = 1.8;

mirrors = [-3 -2.5 -2 -1.5 -1 -0.5 1 1.5 2 2.5 3 inf -.25 .25];
%mirrors = [-3 1 3];
nM = length(mirrors);

%% Build a mirror list
mirrorMatrix = zeros(nM^N, N);
for ii = 1:length(mirrorMatrix)
    for jj = 1:N
       
        idx = mod(fix(ii/nM^(jj-1)), nM) +1;
        mirrorMatrix(ii,jj) = mirrors(idx);
    end
end

TelList = [];
opts = optimset('MaxFunEvals', 1e7, 'TolX', 1e-5,   'TolFun',  1e-7);
xFit = ones(1,N);
for mm = 1:size(mirrorMatrix,1)
    
    thisM = mirrorMatrix(mm,:);
    MyMinFunc = @(x) ...
         1-etaQ(q2, newQ(q1, MultiMirrorTel([x], thisM)));
     
     % Do the minimization with using the previous answer as a first guess
     if(any(xFit<0))
         xFit = ones(1,N);
     end
    [xFit, minval] = fminsearch(MyMinFunc, xFit, opts);
    ABCD = MultiMirrorTel([xFit], thisM);
    zout = OutputRayleigh(q1/1i,  ABCD);
    %TelList(end+1,:) = [x1 xFit thisM minval zout];
    if (minval < 2e-2 & xFit > xMin  & xFit < xMax)
      %  disp('Found one!');
      
        gPhase = TweenGouy(xFit, thisM, q1);
        TelList(end+1,:) = ...
              [xFit  thisM zout gPhase];
     end
 end

% Now that we have a complete list of Telescope parts,  start selecting
% from it.

if  length(TelList) > 0
    % This selection makes sure the second mirror spacing is large enough (and
    % small enough) and sorts the list according to gouy phase
    idx = find( all(TelList(:,1:N)>0.1, 2) & all(TelList(:,1:N) < xMax, 2) );
    TelList= TelList(idx,:);
    [y,idx] = sort(-TelList(:,end));
    TelList = TelList(idx,:);

end

end


function out = OutputRayleigh(zr, ABCD)
    q = 1i*zr;
    out = -1i * newQ(q, ABCD);
end


