function TelList = MakeTelescopeList(q1, zRng);
% function TelList = MakeTelescopeList(q1, zRng);
%
% make a list of all the two mirror telescopes with an output waist at the
% second optic lying between zRng(1) and zRng(2)

if ~exist('q1', 'var')
    q1 = 2.69-7.27i;    % Appropriate for REFL
end

if ~exist('zRng', 'var')  
    zRng = [10 20];
end

mirrors = [-3 -2.5 -2 -1.5 -1 -0.5 -0.25 0.25 0.5 1 1.5 2 2.5 3 inf ];


mirrors = [-3 -2.5 -2 -1.5 -1 -0.5 1 1.5 2 2.5 3 inf -.25 .25];
%mirrors = [-3 1 3];
nM = length(mirrors);
N = 2;
%% Build a mirror list
mirrorMatrix = zeros(nM^N, N);
for ii = 1:length(mirrorMatrix)
    for jj = 1:N
       
        idx = mod(fix(ii/nM^(jj-1)), nM) +1;
        mirrorMatrix(ii,jj) = mirrors(idx);
    end
end

% Now that we have a complete list of Telescope parts,  start selecting
% from it.
opts = optimset('MaxFunEvals', 1e7, 'TolX', 1e-5,   'TolFun',  1e-7);
xMin = 0.1;
xMax = 1.8;
xFit = .5;
TelList = [];
for mm = 1:length(mirrorMatrix)
    thisM = mirrorMatrix(mm,:);
    MyMinFunc = @(x) ...
         abs(real(newQ(q1, TwoMirrorTel(x, thisM(1), thisM(2)))));
     
     % Do the minimization with using the previous answer as a first guess
    [xFit, minval] = fminbnd(MyMinFunc, 0.1, 3);
    ABCD = TwoMirrorTel(xFit, thisM(1), thisM(2));
    qout = newQ(q1,  ABCD);
    %TelList(end+1,:) = [ xFit thisM minval qout];
    if (minval <1e-5 & xFit > xMin  & xFit < xMax)
      %  disp('Found one!');
      
        %gPhase = TweenGouy(xFit, thisM, q1);
        TelList(end+1,:) = ...
              [xFit  thisM imag(qout) ];
     end
 end


% This selection makes sure the second mirror spacing is large enough (and
% small enough) and sorts the list according to gouy phase
%TelList= TelList(TelList(:,2)>0.3 & TelList(:,2) < 1.8,:);
%TelList(:,6) = mod(TelList(:,6), 180)
%[y,i] = sort(-TelList(:,6));
%TelList = TelList(i,:);



end


function ABCD = TwoMirrorTel(dx,r1,r2)

ABCD = [1 0; -2/r2 1] * [1 dx; 0 1] * [1 0; -2/r1 1];

end


function out = OutputRayleigh(q, ABCD)
    out = imag(newQ(q, ABCD));
end

function qout = newQ(q, ABCD)
    qout = (ABCD(1,1)*q + ABCD(1,2)) / (ABCD(2,1)*q + ABCD(2,2));
end

function g = TweenGouy(x1,x2,r1,r2, zr)
% Calculates the Gouy phase between the two mirrors

ABCD = [1 0 ; -2/r1 1] * [1 x1; 0 1];
qin = 1i*zr;
q1 = newQ(qin,ABCD);
ABCD = [1 x2; 0 1];
q2 = newQ(q1, ABCD);
g = 180*(angle(q2) - angle(q1))/pi;

end
