function TelList = MakeThreeTelescope(z1, z2)


if ~exist('zr', 'var')
    z1 = 0.37;
end

if ~exist('z2', 'var')
    z2 = pi*3.3e-3^2/1.064e-6;  % right for POP
end

mirrors = [-3 -2.5 -2 -1.5 -1 -0.5 1 1.5 2 2.5 3 inf];  

q1 = 1i*z1;
q2 = 1i*z2;
TelList = [];
x1 = z1 + 0.1;
for ii = mirrors
    fprintf(1, '%d\n', ii);
    for jj = mirrors
        for kk = mirrors
        
            MyMinFunc = @(x) ...
                1-etaQ(q2, newQ(q1, ThreeMirrorTel(x1, x(1), x(2), ...
                                                   ii, jj, kk)));
            [xmin, minval] = fminsearch(MyMinFunc, [1 1]);
            if (abs(minval) < 1e-2 & xmin > 0)
                ABCD = ThreeMirrorTel(x1, xmin(1), xmin(2), ...
                    ii, jj, kk);
                zout = OutputRayleigh(z1,  ABCD);
                gphase = TweenGouy(x1,xmin(1), xmin(2), ...
                    ii, jj, kk, z1);
            TelList(end+1,:) = ...
                [x1, xmin(1), xmin(2), ...
                ii, jj, kk, real(zout), gphase ];
            end
        end
    end
end

% Now that we have a complete list of Telescope parts,  start selecting
% from it.

% This selection makes sure the second mirror spacing is large enough (and
% small enough) and sorts the list according to gouy phase
TelList= TelList(TelList(:,2)>0.3 & TelList(:,2) < 1.8,:);
%TelList(:,6) = mod(TelList(:,6), 180)
[y,i] = sort(-TelList(:,6));
TelList = TelList(i,:);



end

function ABCD = ThreeMirrorTel(x1,x2,x3, r1, r2, r3)    
    dist = @(x) [1 x; 0 1];
    mirror = @(r) [1 0; -2/r 1];
    
    ABCD = dist(x1) * mirror(r1) * dist(x2) * mirror(r2) * dist(x3) * mirror(r3);
    
end

function ABCD = TwoMirrorTel(x1,x2,r1,r2)

ABCD = [1-2 * x2 / r1,            x2+x1*(1-2*x2/r1);
    -2/r2 - 2/r1 * (1-2*x2/r2),   1-2*x2/r2-2*x1/r2 - 2*x1/r1 *( 1-2*x2/r2)];

end

function out = DesignEq(zr, ABCD)

    out = ABCD(1,1) * ABCD(2,1) * zr^2 + ABCD(1,2) * ABCD(2,2);
    
end

function out = OutputRayleigh(zr, ABCD)
    q = 1i*zr;
    out = -1i * newQ(q, ABCD);
end

function qout = newQ(q, ABCD)
    qout = (ABCD(1,1)*q + ABCD(1,2)) / (ABCD(2,1)*q + ABCD(2,2));
end

function g = TweenGouy(x1,x2, x3, r1,r2, r3, zr)
% Calculates the Gouy phase between the two mirrors

ABCD = [1 0 ; -2/r1 1] * [1 x1; 0 1];
qin = 1i*zr;
q1 = newQ(qin,ABCD);
ABCD = [1 x2; 0 1];
q2 = newQ(q1, ABCD);
g = 180*(angle(q2) - angle(q1))/pi;

end
