% Setup as port

function out = setupAss

% mirror list
mirrors = [-6 -5 -4 -3 -2 -1 -0.5 0.5 1 2 3 4 5 6 inf];
nM = length(mirrors);

% combo list
mirrorCombos = [];
for ii = 1:nM
    for jj = 1:nM
        mirrorCombos(end+1,:) = [mirrors(ii) mirrors(ii) mirrors(jj)];
        if jj ~= ii
            mirrorCombos(end+1,:) = [mirrors(ii) mirrors(jj) mirrors(ii)];
        end
    end
end

%input q
q_in = -.455 + 1.99i;
w_out = 350e-6;
lambda = 1.064e-6;
q_out = 1i*pi*w_out^2/lambda;

R = [];

xGuess = [1 1 1];
xFit = [];
for ii = 1:length(mirrorCombos)
    R = mirrorCombos(ii,:);
   minFunc = @(x) (1-etaQ(q_out, newQ(q_in, ABCD(x, R))));
   xFit(end+1,:) = fminsearch(minFunc, xGuess);
    
end

out.MirrorCombos = mirrorCombos;
out.xFit = xFit;

end

function out = ABCD(dx, R)
% abcd matrix of dx(3) R(3) dx(2) R(2) dx(1) R(1)

dist = @(x) [1 x; 0 1];
mirr = @(r) [1 0; -2/r 1];

out = dist(dx(3)) *  mirr(R(3)) * dist(dx(2)) * mirr(R(2)) * dist(dx(1)) * mirr(R(1));
end

function out = newQ(q_in, ABCD)
% propagate a q parameter
out = (ABCD(1,1)*q_in + ABCD(1,2)) / (ABCD(2,1)*q_in + ABCD(2,2));

end


function eta = etaQ(q1, q2)
% function eta = etaQ(q1, q2)
% 
% Returns the overlap integral of two gaussian beams with q1 and q2.
q1s = conj(q1);
q2s = conj(q2);

s12 = q1 .* q2s ./ (q1 - q2s);
s21 = q2 .* q1s ./ (q2 - q1s);
s11 = q1 .* q1s ./ (q1 - q1s);
s22 = q2 .* q2s ./ (q2 - q2s);


eta = s12 .* s21 ./ (s11 .* s22);

end
