% Setup as port

function out = setupRefl
% We have to cut a mirror to get a good solution,  so let's demand a good
% mirror.  We'll do it by starting with one of the AS mirrors and a fixed
% spacing of 1.0 m.  Then we'll vary the ROC until we get what we want.  

% mirror list
mirrors = [4 1.8];
nM = length(mirrors);

if ~exist('N','var')
    N = 2;
end

mirrorMatrix = zeros(nM^N, N);
for ii = 1:length(mirrorMatrix)
    for jj = 1:N
    
        idx = mod(fix(ii/nM^(jj-1)), nM) +1;
        mirrorMatrix(ii,jj) = mirrors(idx);
    end
end

%input q
q_in = 7.27+2.70i;
w_out = 2e-3;
lambda = 1.064e-6;
q_out = 1i*pi*w_out^2/lambda;

xGuess = ones(N-1,1);
xFit = [];
xGouy = [];
for ii = 1:length(mirrorMatrix)
   R = mirrorMatrix(ii,:);
   minFunc = @(x) (1-etaQ(q_out, newQ(q_in, ABCD(x, R))));
   xFit(end+1,:) = fminsearch(minFunc, xGuess);
   xGouy(end+1, :) = GouyPhases(xFit(end,:), R, q_in);
    
end

out.MirrorMatrix = mirrorMatrix;
out.xFit = xFit;
out.Gouy = xGouy;

end

function out = ABCD(dx, R)
% abcd matrix of dx(3) R(3) dx(2) R(2) dx(1) R(1)

dist = @(x) [1 x; 0 1];
mirr = @(r) [1 0; -2/r 1];
out = mirr(R(1));
for ii = 1:length(dx)
    out = mirr(R(ii+1)) * dist(dx(ii)) * out;
end

  
end

function out = GouyPhases(dx, R, q_in)
% measures the gouy phases between optics with Radius R

dist = @(x) [1 x; 0 1];
mirr = @(r) [1 0; -2/r 1];

q = q_in;
for ii = 1:length(dx)
    q = newQ(q, mirr(R(ii)));
    a1 = angle(q);
    q = newQ(q, dist(dx(ii)));
    a2 = angle(q);
    
    out(ii) = 180/pi*(a1-a2);
end
    
end

    

function out = newQ(q_in, ABCD)
% propagate a q parameter
out = (ABCD(1,1)*q_in + ABCD(1,2)) / (ABCD(2,1)*q_in + ABCD(2,2));

end


function eta = etaQ(q1, q2)
% function eta = etaQ(q1, q2)
% 
% Returns the overlap integral of two gaussian beams with q1 and q2.
q1s = conj(q1);
q2s = conj(q2);

s12 = q1 .* q2s ./ (q1 - q2s);
s21 = q2 .* q1s ./ (q2 - q1s);
s11 = q1 .* q1s ./ (q1 - q1s);
s22 = q2 .* q2s ./ (q2 - q2s);


eta = s12 .* s21 ./ (s11 .* s22);

end
