function result = testPRM(param, q_in)
% function [omega, z0, dGouy] = testPRM(param, q_in)
%
% param is R1, R2, R3, L1, L2
%  for three mirrors of radius R,  spaced by L
% q_in  is the input beam
%
% returns
%   result(1) = output waist size
%   result(2) = distance of the waist from the output 
%   result(3) = the Gouy phase difference between the mirrors 2 and 3
    
mirror = @(R) [1 0; -2/R 1];
mLength = @(tL) [1 tL; 0 1];

% par = [R1 R2 R3 L1 L2];
myTel = @(par) mirror(par(3)) * mLength(par(5)) * mirror(par(2)) * mLength(par(4)) * mirror(par(1));
afterm2 = @(par) mirror(par(2)) *  mLength(par(4)) * mirror(par(1));
beforem3 = @(par) mLength(par(5)) * mirror(par(2)) *  mLength(par(4)) * mirror(par(1));

newQ = @(q, ABCD) (ABCD(1,1)*q + ABCD(1,2))/(ABCD(2,1)*q + ABCD(2,2));

qOut = newQ(q_in, myTel(param));
omega = sqrt(imag(qOut)*1.064e-6/pi);
z0 = real(qOut);

dGouy = angle(newQ(q_in, beforem3(param))) - angle(newQ(q_in, afterm2(param)));
dGouy = dGouy * 180/pi;

result = [omega z0 dGouy];


